searchNodes=[{"ref":"Solution.html","title":"Solution","type":"module","doc":"A Macro-based approach to working with ok/error tuples This module exposes two main things: guard-clause macros is_ok/1, is_error/1 and is_okerror/1 (as well as arity-2 variants of the same), to be used whenever you like. scase/2 and swith/2, replacements for case and with-statements respectively that allow you to pattern match on ok/error tuples more effectively, as well as bind to one ore multiple of the values stored inside."},{"ref":"Solution.html#error/0","title":"Solution.error/0","type":"macro","doc":"Matches any error datatype. (See also is_error/1) Has to be used inside the LHS of a scase or swith statement."},{"ref":"Solution.html#error/1","title":"Solution.error/1","type":"macro","doc":"Matches {:error, res} (as well as tuples with more elements). res is then bound. (See also is_error/1) Has to be used inside the LHS of a scase or swith statement."},{"ref":"Solution.html#from_nillable/1","title":"Solution.from_nillable/1","type":"function","doc":"Turns a nillable type (that can be either nil or a non-nil value) into an ok/error tuple. Also handles Erlang&#39;s &#39;nil&#39;-type equivalent: the atom :undefined. iex&gt; from_nillable(nil) {:error, nil} iex&gt; from_nillable(42) {:ok, 42} iex&gt; (%{a: &quot;yes!&quot;} |&gt; Map.get(:a) |&gt; from_nillable()) {:ok, &quot;yes!&quot;} iex&gt; (%{a: &quot;yes!&quot;} |&gt; Map.get(:b) |&gt; from_nillable()) {:error, nil} iex&gt; from_nillable(:undefined) {:error, :undefined}"},{"ref":"Solution.html#invert_okerror/1","title":"Solution.invert_okerror/1","type":"function","doc":"Changes an :ok into an :error, an {:ok, ...} into an {:error, ...} and vice-versa. iex&gt; invert_okerror(:ok) :error iex&gt; invert_okerror({:ok, 1,2,3}) {:error, 1,2,3} iex&gt; invert_okerror({:error, &quot;failure&quot;}) {:ok, &quot;failure&quot;} iex&gt; invert_okerror(&quot;improper datatype&quot;) ** (ArgumentError) argument error"},{"ref":"Solution.html#is_error/1","title":"Solution.is_error/1","type":"macro","doc":"Matches when x is one of the following: :error {:error, _} {:error, _, _} or a longer tuple where the first element is the atom :error. ({:error, ...}) iex&gt; is_error(:error) true iex&gt; is_error({:error, 42}) true iex&gt; is_error({:error, &quot;I&quot;, &quot;have&quot;, &quot;many&quot;, &quot;elements&quot;}) true iex&gt; is_error(:asdf) false iex&gt; is_error({:ok, &quot;success!&quot;}) false"},{"ref":"Solution.html#is_error/2","title":"Solution.is_error/2","type":"macro","doc":"Matches when x is a long-enough ok-tuple that has more than n_elems elements."},{"ref":"Solution.html#is_ok/1","title":"Solution.is_ok/1","type":"macro","doc":"Matches when x is one of the following: :ok {:ok, _} {:ok, _, _} or a longer tuple where the first element is the atom :ok ({:ok, ...}) iex&gt; is_ok(:ok) true iex&gt; is_ok({:ok, 42}) true iex&gt; is_ok({:ok, &quot;I&quot;, &quot;have&quot;, &quot;many&quot;, &quot;elements&quot;}) true iex&gt; is_ok(:asdf) false iex&gt; is_ok({:error, &quot;failure&quot;}) false"},{"ref":"Solution.html#is_ok/2","title":"Solution.is_ok/2","type":"macro","doc":"Matches when x is a long-enough ok-tuple that has more than n_elems elements."},{"ref":"Solution.html#is_okerror/1","title":"Solution.is_okerror/1","type":"macro","doc":"Matches when either is_ok(x) or is_error(x) matches. iex&gt; is_okerror({:ok, &quot;Yay!&quot;}) true iex&gt; is_okerror({:error, &quot;Nay&quot;}) true iex&gt; is_okerror(false) false iex&gt; is_okerror({}) false iex&gt; is_okerror({:ok, &quot;the&quot;, &quot;quick&quot;, &quot;brown&quot;, &quot;fox&quot;}) true"},{"ref":"Solution.html#is_okerror/2","title":"Solution.is_okerror/2","type":"macro","doc":"Matches when x is a long-enough ok-tuple that has more than n_elems elements. Warning: Will not match plain :ok or :error!"},{"ref":"Solution.html#ok/0","title":"Solution.ok/0","type":"macro","doc":"Matches any ok datatype. (See also is_ok/1) Has to be used inside the LHS of a scase or swith statement."},{"ref":"Solution.html#ok/1","title":"Solution.ok/1","type":"macro","doc":"Matches {:ok, res} (as well as tuples with more elements). res is then bound. (See also is_ok/1) Has to be used inside the LHS of a scase or swith statement."},{"ref":"Solution.html#okerror/0","title":"Solution.okerror/0","type":"macro","doc":"Matches any ok/error datatype. Has to be used inside the LHS of a scase or swith statement."},{"ref":"Solution.html#okerror/1","title":"Solution.okerror/1","type":"macro","doc":"Matches any ok or error type. tag is then bound to :ok or :error. (See also is_okerror/1) Has to be used inside the LHS of a scase or swith statement."},{"ref":"Solution.html#okerror/2","title":"Solution.okerror/2","type":"macro","doc":"Matches {:ok, res}, {:error, res} (as well as tuples with more elements). tag and res are bound. (See also is_okerror/1) tag is bound to the value :ok or :error depending on the tuple. res is bound to what the second element might be. Has to be used inside the LHS of a scase or swith statement."},{"ref":"Solution.html#scase/2","title":"Solution.scase/2","type":"macro","doc":"Works like a normal case-statement, but will expand ok(), error() and okerror()macros to the left side of -&gt;. iex&gt; scase {:ok, 10} do ...&gt; ok() -&gt; &quot;Yay!&quot; ...&gt; _ -&gt; &quot;Failure&quot; ...&gt; end &quot;Yay!&quot; You can also pass arguments to ok(), error() or okerror() which will then be bound and available to be used inside the case expression: iex&gt; scase {:ok, &quot;foo&quot;, 42} do ...&gt; ok(res, extra) -&gt; ...&gt; &quot;result: \#{res}, extra: \#{extra}&quot; ...&gt; _ -&gt; ...&gt; &quot;Failure&quot; ...&gt; end &quot;result: foo, extra: 42&quot; Note that for ok() and error(), the first argument will match the first element after the :ok or :error tag. On the other hand, for okerror(), the first argument will match the tag :ok or :error. Note: It is not required to import Solution to use the macros inside swith without prefixing them."},{"ref":"Solution.html#swith/2","title":"Solution.swith/2","type":"macro","doc":"Works like a normal with-statement, but will expand ok(), error() and okerror() macros to the left side of &lt;-. iex&gt; x = {:ok, 10} iex&gt; y = {:ok, 33} iex&gt; swith ok(res) &lt;- x, ...&gt; ok(res2) &lt;- y do ...&gt; &quot;We have: \#{res} \#{res2}&quot; ...&gt; else ...&gt; _ -&gt; &quot;Failure&quot; ...&gt; end &quot;We have: 10 33&quot; You can also pass arguments to ok(), error() or okerror() which will then be bound and available to be used inside the rest of the swith-expression: iex&gt; x = {:ok, 10} iex&gt; y = {:error, 33} iex&gt; z = {:ok, %{a: 42}} iex&gt; swith ok(res) &lt;- x, ...&gt; error(res2) &lt;- y, ...&gt; okerror(tag, metamap) &lt;- z, ...&gt; %{a: val} = metamap do ...&gt; &quot;We have: \#{res} \#{res2} \#{tag} \#{val}&quot; ...&gt; else ...&gt; _ -&gt; &quot;Failure&quot; ...&gt; end &quot;We have: 10 33 ok 42&quot; Note that for ok() and error(), the first argument will match the first element after the :ok or :error tag. On the other hand, for okerror(), the first argument will match the tag :ok or :error. Note: It is not required to import Solution to use the macros inside swith without prefixing them."},{"ref":"Solution.Enum.html","title":"Solution.Enum","type":"module","doc":"Helper functions to work with ok/error tuples in Enumerables."},{"ref":"Solution.Enum.html#combine/1","title":"Solution.Enum.combine/1","type":"function","doc":"Changes a list of oks into {:ok, list_of_values} If any element of the list is an error, returns this error element. If all elements are oks, takes the first non-tag value from each of them for the list. (in the case of :ok, nothing is added to the resulting list.) iex&gt; combine([{:ok, 1}, {:ok, &quot;a&quot;, %{meta: &quot;this will be dropped&quot;}}, {:ok, :asdf}]) {:ok, [1, &quot;a&quot;, :asdf]} iex&gt; combine([{:ok, 1}, {:ok, 2}, {:error, 3}]) {:error, 3} iex&gt; combine([{:ok, 1}, {:ok, 2}, {:error, 3, 4, 5}]) {:error, 3, 4, 5}"},{"ref":"Solution.Enum.html#error_vals/1","title":"Solution.Enum.error_vals/1","type":"function","doc":"Returns a list of the values of all error-type elements in the enumerable. Note that the &#39;value&#39; is taken to be the first element: iex&gt; error_vals([{:error, 1}, {:error, 2,3,4,5}]) [1, 2]"},{"ref":"Solution.Enum.html#error_valstuples/1","title":"Solution.Enum.error_valstuples/1","type":"function","doc":"Returns a list of the tuple-values of all error-type elements in the enumerable. The &#39;tuple-value&#39; consists of all elements in the error-type element, except for the initial :error tag. iex&gt; error_valstuples([{:error, 1}, {:error, 2,3,4,5}]) [{1}, {2,3,4,5}]"},{"ref":"Solution.Enum.html#errors/1","title":"Solution.Enum.errors/1","type":"function","doc":"Returns a list of only all error-type elements in the enumerable. iex&gt; errors([{:ok, 1}, {:error, 2}, {:ok, 3}]) [{:error, 2}]"},{"ref":"Solution.Enum.html#ok_vals/1","title":"Solution.Enum.ok_vals/1","type":"function","doc":"Returns a list of the values of all ok-type elements in the enumerable. Note that the &#39;value&#39; is taken to be the first element: iex&gt; ok_vals([{:ok, 1}, {:ok, 2,3,4,5}]) [1, 2]"},{"ref":"Solution.Enum.html#ok_valstuples/1","title":"Solution.Enum.ok_valstuples/1","type":"function","doc":"Returns a list of the tuple-values of all ok-type elements in the enumerable. The &#39;tuple-value&#39; consists of all elements in the ok-type element, except for the initial :ok tag. iex&gt; ok_valstuples([{:ok, 1}, {:ok, 2,3,4,5}]) [{1}, {2,3,4,5}]"},{"ref":"Solution.Enum.html#oks/1","title":"Solution.Enum.oks/1","type":"function","doc":"Returns a list of only all ok-type elements in the enumerable. iex&gt; oks([{:ok, 1}, {:error, 2}, {:ok, 3}]) [{:ok, 1}, {:ok, 3}]"},{"ref":"readme.html","title":"README","type":"extras","doc":"Solution is a library to help you with working with ok/error-tuples in case and with-expressions by exposing special matching macros, as well as some extra helper functions."},{"ref":"readme.html#installation","title":"README - Installation","type":"extras","doc":"You can install Solution by adding solution to your list of dependencies in mix.exs: def deps do [ {:solution, &quot;~&gt; 0.2.0&quot;} ] end"},{"ref":"readme.html#rationale","title":"README - Rationale","type":"extras","doc":"ok/error tuples, which are also known by many other names some common ones being &#39;Tagged Status&#39; tuples, &#39;OK tuples&#39;, &#39;Success Tuples&#39;, &#39;Result tuples&#39;, &#39;Elixir Maybes&#39;. Working with these types is however a bit complicated, since functions of different libraries (including different approaches in the Elixir standard library and the Erlang standard library) indicate a successful or failure result, in practice, in one of the following formats: {:ok, val} when everything went well {:error reason} when there was a failure. :ok, when everything went well but there is no useful return value to share. :error, when there was a failure bht there is no useful return value to share. {:ok, val, extra} ends up being used by some libraries that want to return two things on success. {:error, val, extra} ends up being used by some libraries that want to return two things on failure. In general, {:ok, ...} or {:error, ...} with more elements have seen some (albeit luckily limited) use. Clearly, a simple pattern match does not cover all of these cases. This is where Solution comes in: It defines clever guard macros that match either of these groups (Solution.is_ok/1, Solution.is_error/1, Solution.is_okerror/1) It defines macros to be used inside special case and with statements that use these guards and are also able to bind variables: For instance, you might use Solution.ok/0 to match any ok-type datatype, and Solution.error/0 to match any error-type datatype. But they will also bind variables for you: So you can use ok(x) to bind x = 42 regardless of whether {:ok, 42}, {:ok, 42, &quot;foo&quot;} or {:ok, 42, 3,1,4,1,5,9,2,6,5} was passed. Higher-arity forms like ok(x, y, z) also exist."},{"ref":"readme.html#examples","title":"README - Examples","type":"extras","doc":"The following example snippets assume that you run require Solution import Solution (although of course, Solution is also fully usable without importing it.) Guards Solution exposes three guard-safe functions: is_ok(x), is_error(x) and is_okerror(x) Solution.is_ok/1 will match :ok, {:ok, _}, {:ok, _, _}, {:ok, _, _, __} and any longer tuple whose first element is :ok. Solution.is_error/1 will match :error, {:error, _}, {:error, _, _}, {:error, _, _, __} and any longer tuple whose first element is :error. Solution.is_okerror/1 matches both of these. Solution also exposes versions of these that take a &#39;minimum-length&#39; as second argument. A length of 0 works jus the same as above versions. Longer lengths only match tuples that have at least that many elements (as well as starting with the appropriate tag). SCase Solution.scase/2works like a normal case-statement, but will expand ok(), error() and okerror()macros to the left side of -&gt;. scase {:ok, 10} do ok() -&gt; &quot;Yay!&quot; _ -&gt; &quot;Failure&quot; end #=&gt; &quot;Yay!&quot; You can also pass arguments to ok(), error() or okerror() which will then be bound and available to be used inside the case expression: scase {:ok, &quot;foo&quot;, 42} do ok(res, extra) -&gt; &quot;result: \\\#{res}, extra: \\\#{extra}&quot; _ -&gt; &quot;Failure&quot; end #=&gt; &quot;result: foo, extra: 42&quot; Note that for ok(...) and error(...), the first argument will match the first element after the :ok or :error tag. On the other hand, for okerror(...), the first argument will match the tag :ok or :error. SWith Solution.swith/2 works like a normal with-statement, but will expand ok(), error() and okerror() macros to the left side of &lt;-. x = {:ok, 10} y = {:ok, 33} swith ok(res) &lt;- x, ok(res2) &lt;- y do &quot;We have: \\\#{res} \\\#{res2}&quot; else _ -&gt; &quot;Failure&quot; end #=&gt; &quot;We have: 10 33&quot; You can also pass arguments to ok(), error() or okerror() which will then be bound and available to be used inside the rest of the swith-expression: x = {:ok, 10} y = {:error, 33} z = {:ok, %{a: 42}} swith ok(res) &lt;- x, error(res2) &lt;- y, okerror(tag, metamap) &lt;- z, %{a: val} = metamap do &quot;We have: \\\#{res} \\\#{res2} \\\#{tag} \\\#{val}&quot; else _ -&gt; &quot;Failure&quot; end #=&gt; &quot;We have: 10 33 ok 42&quot; Note that for ok(...) and error(...), the first argument will match the first element after the :ok or :error tag. On the other hand, for okerror(...), the first argument will match the tag :ok or :error. Converting from Nillable types When functions return a value that can never be the atom nil, they often represent error by returning nil. Erlang also has its own equivalent of nil that is used in many places in the Erlang standard library and other Erlang libraries: :undefined. Solution.from_nillable/1 can be used to convert these types into either {:ok, thing} if thing is nil nor :undefined, and into {:error, nil} or {:error, :undefined} otherwise. Solution.Enum The Solution.Enum module contains helper functions to work with enumerables of ok/error tuples. Solution.Enum.combine/1 Changes a list of oks into {:ok, list_of_values} Solution.Enum.combine([{:ok, 1}, {:ok, &quot;a&quot;, %{meta: &quot;this will be dropped&quot;}}, {:ok, :asdf}]) #=&gt; {:ok, [1, &quot;a&quot;, :asdf]} Solution.Enum.combine([{:ok, 1}, {:ok, 2}, {:error, 3}]) #=&gt; {:error, 3} Solution.Enum.combine([{:ok, 1}, {:ok, 2}, {:error, 3, 4, 5}]) #=&gt; {:error, 3, 4, 5} Solution.Enum.oks/1 Returns a list of only all ok-type elements in the enumerable. Solution.Enum.oks([{:ok, 1}, {:error, 2}, {:ok, 3}]) #=&gt; [{:ok, 1}, {:ok, 3}] Similarly, there also exists errors/1 Solution.Enum.ok_vals/1 Returns a list of the values of all ok-type elements in the enumerable. Solution.Enum.ok_vals([{:ok, 1}, {:ok, 2,3,4,5}]) #=&gt; [1, 2] Similarly, there also exists Solution.Enum.error_vals/1, as well as Solution.Enum.ok_valstuples/1 and Solution.Enum.error_valstuples/1."},{"ref":"readme.html#documentation","title":"README - Documentation","type":"extras","doc":"Full documentation can be found at https://hexdocs.pm/solution."},{"ref":"readme.html#changelog","title":"README - Changelog","type":"extras","doc":"1.0.0 - Release of stable version. No API changes since last version. 0.2.1 - Proper handling of :undefined (like nil). 0.2.0 - the Solution.Enum module was added 0.1.0 - Initial version"}]